@@ -1,16 +1,21 @@
 #!/usr/bin/env python
-"""
-Testing Monitor. 
-This will be included in SimPyTest eventually.
-'$Revision: 1.1.1.1.4.1.4.2 $ $Date: 2005/05/27 05:03:57 $'
-# 2004 05 03 corrected test for Monitored queues (gav)
-"""
-__version__="1.6 June 2005"
 from __future__ import generators
 from random import *
-from Simulation  import *
+from SimPy.Simulation import *
 from SimPy.SimPlot import SimPlot
 import unittest
+# $Revision: 1.1.1.17 $ $Date: 2006/06/14 12:28:24 $
+
+"""MonitorTest.py
+Testing Monitor, Tally. 
+This may be included in SimPyTest eventually.
+
+Change history:
+# 2004 05 03 corrected test for Monitored queues (gav)
+# 2005 09 06 added tests for Tally (kgm)
+"""
+__version__="1.7.1 Jun 2006"
+
 
 ## ------------------------------------------------------------
 class Thing(Process):
@@ -110,8 +115,8 @@ class makeMtestCase(unittest.TestCase):
         assert m.name == 'First','Tally name wrong'
         assert m.total() == 45,'Tally total wrong'
         assert m.mean() == 4.5,'Tally mean wrong'
-        assert m.var()  == (285-(45*45/10.0))/10.0,'Tally sample var wrong%s'%(m.var(),)
-        
+        assert m.var()  == (285-(45*45/10.0))/10.0,'Tally sample var wrong%s'%(m.var(),)       
+
 
     def testAccumulate(self):
         """Test Monitor - accumulate"""
@@ -157,7 +162,7 @@ class makeMtestCase(unittest.TestCase):
        
        
     def testhistogram(self):
-       """Test some Monitor list operations"""
+       """Test Monitor histogram"""
        m = Monitor(name='First')      
        for y in [-5, 0, 5, 15,99,105,120]:m.observe(y)
        h = m.histogram(low=0.0,high=100.0,nbins=10)
@@ -186,75 +191,196 @@ def makeMSuite():
                     ]) 
     return suite
 
+#### -----------------------------------------------------------------------
+##class makeHtestCase(unittest.TestCase):
+##    """ Test Histogram"""
+##        
+##    def setUp(self):
+##        self.H = Histogram(name='First Test',low=0.0,high=100.0,nbins=10)
+##        #print 'setup ',self.H
+##       
+##    def testHistogram(self):
+##        """Test Histogram"""
+##        h = self.H
+##        #print 'h = :',h
+##        assert h.name == 'First Test', 'wrong histogram name: %s'%h.name
+##        #print h.nbins,h.binsize, h
+##        assert h[0]  == [h.low-h.binsize,0], 'wrong init of under bin: %s'%h
+##        i=h.nbins+1
+##        assert h[-1] == [h.low+(i-1)*h.binsize,0], 'wrong init of over bin: %s'%h
+## 
+##    def testaddIn(self):
+##        """Test addIn"""
+##        h=self.H
+##        for y in [-5, 0, 5, 15,99,105,120]:
+##            h.addIn(y)
+##        assert h[0][1]  == 1, 'wrong value for under: %s'%h[0]
+##        assert h[1][1]  == 2, 'wrong value for 1 (low): %s'%h[1]
+##        assert h[2][1]  == 1, 'wrong value for 2 (low + 1)%s'%h[1]
+##        assert h[10][1] == 1, 'wrong value for 10 (high): %s'%h[10]
+##        assert h[11][1] == 2, 'wrong value for 11 (over): %s'%h[11]
+##
+##    def testPlot(self):
+##        """Test plot histogram"""
+##        h=self.H
+##        for y in [-5, 0, 5, 15,15,15,15,15,15,15,15,15,15,99,105,120]:
+##            h.addIn(y)
+##        h2=Histogram(name='long test',low=0, high=100,nbins=100)
+##        seed =1133557799
+##        g = Random(seed)
+##        for i in range(1000):
+##            h2.addIn(g.expovariate(1.0/33))
+##   
+##
+##        plt=SimPlot()
+##        plt.plotStep(h,
+##             title='Basic step plot',
+##                     color='green',width=3,
+##                     xaxis='minimal',
+##                     yaxis='minimal')
+##        plt.plotStep(h2,
+##                     color='blue',
+##                     #xaxis='minimal',
+##                     yaxis='minimal')
+##        plt.mainloop()
+##
+
+##def makeHSuite():
+##    suite = unittest.TestSuite()
+##    testHistogram = makeHtestCase("testHistogram")
+##    testaddIn = makeHtestCase("testaddIn")
+##    testPlot = makeHtestCase("testPlot")
+##    suite.addTests([testHistogram,
+##                    testPlot,
+##                    testaddIn,
+##                    ]) 
+##    return suite
+    
 ## -----------------------------------------------------------------------
-class makeHtestCase(unittest.TestCase):
-    """ Test Histogram"""
+## Tally test cases
+## -----------------------------------------------------------------------
+class makeTtestCase(unittest.TestCase):
+    """ Test Tally
+    """
         
     def setUp(self):
-        self.H = Histogram(name='First Test',low=0.0,high=100.0,nbins=10)
-        #print 'setup ',self.H
+        self.T = Tally(name='First')
+        for i in range(10):
+            self.T.observe(2*i,i)
+        self.TT = Tally(name='tallier')
+
+    def testObserve(self):
+        """Test Tally - observe"""
+        t = self.T
+        #for i in m.series():
+        #   print i
+        assert t.name == 'First','name wrong'
+        assert t.total() == 90, 'total wrong:%s'%m.total()
+        assert t.mean() == 9.0,'mean wrong:%s'%m.mean()
+        assert t.var() == (4*285.-(90*90/10.0))/10.0,'sample var wrong:%s'%(t.var(),)
+
+    def testObserveNoTime(self):
+        """Test Tally - observe with time being picked up from now()"""
+        ta= Tally(name='No time')
+        initialize()
+        t = Thing(ta)
+        activate(t,t.execute(),0.0)
+        simulate(until=20.0)
+        assert ta.total() == 15, 'total wrong:%s'%ta.total()
+        assert ta.timeAverage(10.0) == 5.0 ,'time average is wrong:%s'%ta.timeAverage(10.0)
+
+    def testtimeAverage(self):
+       """ test time averages """
+       ta= self.T
+       assert ta.timeAverage(10.0) == 9.0 ,'time average is wrong:%s'%ta.timeAverage(10.0)
+    
+    def testreset(self):
+       """ test time averages """
+       ta=self.T
+       ta.reset(t = 10.0)
+       assert ta.startTime == 10.0,'reset time  wrong'
        
-    def testHistogram(self):
-        """Test Histogram"""
-        h = self.H
-        #print 'h = :',h
-        assert h.name == 'First Test', 'wrong histogram name: %s'%h.name
-        #print h.nbins,h.binsize, h
-        assert h[0]  == [h.low-h.binsize,0], 'wrong init of under bin: %s'%h
-        i=h.nbins+1
-        assert h[-1] == [h.low+(i-1)*h.binsize,0], 'wrong init of over bin: %s'%h
- 
-    def testaddIn(self):
-        """Test addIn"""
-        h=self.H
-        for y in [-5, 0, 5, 15,99,105,120]:
-            h.addIn(y)
-        assert h[0][1]  == 1, 'wrong value for under: %s'%h[0]
-        assert h[1][1]  == 2, 'wrong value for 1 (low): %s'%h[1]
-        assert h[2][1]  == 1, 'wrong value for 2 (low + 1)%s'%h[1]
-        assert h[10][1] == 1, 'wrong value for 10 (high): %s'%h[10]
-        assert h[11][1] == 2, 'wrong value for 11 (over): %s'%h[11]
+    def testhistogram(self):
+       """Test some Monitor list operations"""
+       ta = Monitor(name='First')      
+       for y in [-5, 0, 5, 15,99,105,120]:ta.observe(y)
+       ta.setHistogram(low=0.0,high=100.0,nbins=10)
+       h = ta.histogram()
+       shouldBe = list(zip(*h)[1])
+       assert shouldBe == [1,2,1,0,0,0,0,0,0,0,1,2], 'm histogram is wrong: %s'%(shouldBe,)
 
-    def testPlot(self):
-        """Test plot histogram"""
-        h=self.H
-        for y in [-5, 0, 5, 15,15,15,15,15,15,15,15,15,15,99,105,120]:
-            h.addIn(y)
-        h2=Histogram(name='long test',low=0, high=100,nbins=100)
-        seed =1133557799
-        g = Random(seed)
-        for i in range(1000):
-            h2.addIn(g.expovariate(1.0/33))
-   
-
-        plt=SimPlot()
-        plt.plotStep(h,
-             title='Basic step plot',
-                     color='green',width=3,
-                     xaxis='minimal',
-                     yaxis='minimal')
-        plt.plotStep(h2,
-                     color='blue',
-                     #xaxis='minimal',
-                     yaxis='minimal')
-        plt.mainloop()
-
-
-def makeHSuite():
+def makeTSuite():
     suite = unittest.TestSuite()
-    testHistogram = makeHtestCase("testHistogram")
-    testaddIn = makeHtestCase("testaddIn")
-    testPlot = makeHtestCase("testPlot")
-    suite.addTests([testHistogram,
-                    testPlot,
-                    testaddIn,
+    testObserve = makeTtestCase("testObserve")
+    testObserveNoTime = makeTtestCase("testObserveNoTime")
+    testtimeAverage = makeTtestCase("testtimeAverage")
+    testreset = makeTtestCase("testreset")
+    testhistogram = makeTtestCase("testhistogram")
+    suite.addTests([testObserve,testObserveNoTime,
+                    testtimeAverage,
+                    testreset,
+                    testhistogram,
+                    ]) 
+    return suite
+    
+## -----------------------------------------------------------------------
+## Test cases to test equivalence of Monitor and Tally
+## for monitored Resource instances
+## -----------------------------------------------------------------------
+
+class Actor(Process):
+    """Process used in MakeEquivTestCase"""
+    def act(self,res):
+        while True:
+            yield request,self,res
+            yield hold,self,1
+            yield release,self,res
+            
+class makeEquivTestCase(unittest.TestCase):
+    """To test that the histograms produced in monitoring
+    a Resource instance's queues are equivalent
+    """
+    def testResHistogram(self):
+        initialize()
+        r=Resource(monitored=True,monitorType=Monitor,name="TheResource/Monitor")
+        r.waitMon.setHistogram(high=3,nbins=3)
+        r.actMon.setHistogram(high=3,nbins=3)
+
+        for i in range (5):
+          a=Actor()
+          activate(a,a.act(r))
+        simulate(until=20)
+        mHistoAct= r.actMon.getHistogram()
+        mHistoWait=r.waitMon.getHistogram()
+
+        initialize()
+        r=Resource(monitored=True,monitorType=Tally,name="TheResource/Tally")
+        r.waitMon.setHistogram(high=3,nbins=3)
+        r.actMon.setHistogram(high=3,nbins=3)
+        for i in range (5):
+          a=Actor()
+          activate(a,a.act(r))
+        simulate(until=20)
+        tHistoAct= r.actMon.getHistogram()
+        tHistoWait=r.waitMon.getHistogram()
+
+        assert mHistoAct==tHistoAct,"actMon histograms are different"
+        assert mHistoWait==tHistoWait,"waitMon histograms are different"
+
+def makeEquivSuite():
+    suite = unittest.TestSuite()
+    testResHistogram = makeEquivTestCase("testResHistogram")
+    suite.addTests([testResHistogram
                     ]) 
     return suite
 
+## -----------------------------------------------------------------------
+
 if __name__ == '__main__':
     alltests = unittest.TestSuite((makeMSuite(),
-                                   makeHSuite(),
-                                   #makeIntegraterSuite(),
+##                                   makeHSuite(),
+                                   makeTSuite(),
+                                   makeEquivSuite()
                                    ))
     runner = unittest.TextTestRunner()
     runner.run(alltests)
