@@ -1,8 +1,6 @@
 #!/usr/bin/env python
-"""SimGUI 1.6
-__version__ = '$Revision: 1.1.1.1.4.2.4.1 $ $Date: 2005/05/27 05:17:26 $ kgm'
-
-SimGUI implements a Tk/Tkinter-based framework for SimPy simulation
+# $Revision: 1.1.1.8 $ $Date: 2006/06/14 12:28:24 $ kgm
+"""SimGUI 1.7.1  Provides a Tk/Tkinter-based framework for SimPy simulation
 models.
 
 LICENSE:
@@ -41,8 +39,7 @@ developed by Ka-Ping Yee, <ping@lfw.org>.
 
     February 2004: Release as part of SimPy 1.4
     
-    '$Revision: 1.1.1.1.4.2.4.1 $ $Date: 2005/05/27 05:17:26 $ kgm'
-    """
+"""
 
 from __future__ import generators
 from Tkinter import *
@@ -50,7 +47,7 @@ from tkMessageBox import *
 from Canvas import Line, CanvasText, Rectangle
 import tkconsole as tkcons
 
-__version__ = '1.6 June 2005'
+__version__ = '1.7.1 Jun 2006'
 
 class SimGUI(object):
     def __init__(self,win,title="SimGUI",doc="No doc string found",consoleHeight=50):
