@@ -1,7 +1,6 @@
 #!/usr/bin/env python
-""" SimPlot 1.6
-
-$Revision: 1.1.1.1.4.2.4.1 $ $Date: 2005/05/27 05:17:13 $ kgm
+# $Revision: 1.1.1.13 $ $Date: 2006/06/14 12:28:24 $ kgm
+""" SimPlot 1.7.1  Provides basic plotting services based on Tk/Tkinter.
 
 LICENSE:
 Copyright (C) 2002, 2005  Klaus G. Muller, Tony Vignaux
@@ -22,9 +21,7 @@ mailto: kgmuller@xs4all.nl and Tony.Vignaux@vuw.ac.nz
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 END OF LICENSE
 
-SimPlot provides basic plotting services based on Tk/Tkinter.
 Derived from plotting package in Grayson's Tkinter book.
-
 The idea to use this package came from Prof. Simon Frost
 of U of California, San Diego who also strongly contributed
 to the design and implementation of SimPlot.
@@ -36,9 +33,10 @@ Change history:
     Nov 11, 2003: Made totally OO
     Dec 16, 2003: Completion of SimPlot 1.4alpha
     Feb 2004:       Release with SimPy 1.4
+    Aug 27, 2005: Added tests for empty point sets to plotXXX functions.
 
 """
-__version__= '1.6 June 2005'
+__version__= '1.7.1 Jun 2006'
 from Tkinter import *
 from Canvas import Line, CanvasText, Rectangle
 from tkMessageBox import *
@@ -156,135 +154,162 @@ class SimPlot(object):
     def plotLine(self,points,windowsize=(500,300),title='',width=1,color='black',
                  smooth=0,background='white',xlab='x',ylab='y',
                  xaxis='automatic',yaxis='automatic'):
-        root=Toplevel()
-        f=Frame(root)
-        try: #if it is like a Monitor, take xlab,ylab from it
-            ylab=points.ylab
-            xlab=points.tlab
-            if not title: title=points.name 
-        except:
-            pass
-        line=self.makeLine(points, width=width,color=color,smooth=smooth)
-        gr=self.makeGraphObjects([line])
-        graph = self.makeGraphBase(f, windowsize[0], windowsize[1],
+        """Generates a line chart, with menu to save as Postscript file.
+        'points' can be a Monitor instance.
+        """
+        if points!=[]:
+            root=Toplevel()
+            f=Frame(root)
+            try: #if it is like a Monitor, take xlab,ylab from it
+                ylab=points.ylab
+                xlab=points.tlab
+                if not title: title=points.name 
+            except:
+                pass
+            line=self.makeLine(points, width=width,color=color,smooth=smooth)
+            gr=self.makeGraphObjects([line])
+            graph = self.makeGraphBase(f, windowsize[0], windowsize[1],
                                    title=title,xtitle=xlab,
                                    ytitle=ylab,background=background)
-        graph.pack(side=LEFT, fill=BOTH, expand=YES)
-        graph.draw(gr,xaxis=xaxis, yaxis=yaxis)
-        #File menu
-        self.graphMenu(root,graph)
-        f.pack()
-        return graph
+            graph.pack(side=LEFT, fill=BOTH, expand=YES)
+            graph.draw(gr,xaxis=xaxis, yaxis=yaxis)
+            #File menu
+            self.graphMenu(root,graph)
+            f.pack()
+            return graph
+        else:
+            print "SimPlot.plotline: dataset empty, no plot."
+            return None
     
     def plotStep(self,points,windowsize=(500,300),title='',width=1,color='black',
                  background='white',xlab='x',ylab='y',
                  xaxis='automatic',yaxis='automatic'):
-        #convert data list to steps
-        step0=points[:]
-        step1=[[0,0]]*2*len(step0)
-        prev=[step0[0][0],0]
-        for x in range(len(step0)):
-            step1[2*x]=[step0[x][0],prev[1]]
-            step1[2*x+1]=step0[x]
-            prev=step0[x]
-        #treat monitor case
-        try: #if it is like a Monitor, take xlab,ylab from it
-            ylab=points.ylab
-            xlab=points.tlab
-            if not title: title=points.name 
-        except:
-            pass
-        #draw the line
-        smooth=False
-        return self.plotLine(step1,windowsize,title,width,color,
+        """Generates a step chart, with menu to save as Postscript file.
+        'points' can be a Monitor instance.
+        """
+        if points!=[]:
+            #convert data list to steps
+            step0=points[:]
+            step1=[[0,0]]*2*len(step0)
+            prev=[step0[0][0],0]
+            for x in range(len(step0)):
+                step1[2*x]=[step0[x][0],prev[1]]
+                step1[2*x+1]=step0[x]
+                prev=step0[x]
+            #treat monitor case
+            try: #if it is like a Monitor, take xlab,ylab from it
+                ylab=points.ylab
+                xlab=points.tlab
+                if not title: title=points.name 
+            except:
+                pass
+            #draw the line
+            smooth=False
+            return self.plotLine(step1,windowsize,title,width,color,
                  smooth,background,xlab,ylab,
                  xaxis,yaxis)
+        else:
+            print "SimPlot.plotStep: dataset empty, no plot."
+            return None
 
     def plotHistogram(self,points,windowsize=(500,300),title='',width=1,color='black',
                  background='white',xlab='x',ylab='y',
                  xaxis='automatic',yaxis='automatic'):
-        """Makes a histogram plot. 'points' must be a Histogram-like
-        object.
+        """Makes a histogram plot. 'points' can be a Monitor  instance.
         """
-        #convert data list to bars
-        step0=points[:]
-        step1=[[0,0]]*3*len(step0)
-        prev=[step0[0][0],0]
-        for x in range(len(step0)):
-            step1[3*x]=[step0[x][0],prev[1]]
-            step1[3*x+1]=[step0[x][0],0.0]
-            step1[3*x+2]=step0[x]
-            prev=step0[x]
-        deltax=step0[1][0]-step0[0][0]
-        step1.append([prev[0]+deltax,prev[1]])
-        step1.append([prev[0]+deltax,0])
-        #treat monitor case
-        try: #if it is like a Monitor, take xlab,ylab from it
-            ylab=points.ylab
-            xlab=points.tlab
-            if not title: title=points.name 
-        except:
-            pass
-        #draw the line
-        smooth=False
-        return self.plotLine(step1,windowsize=windowsize,title=title,width=width,
+        if points!=[]:
+            #convert data list to bars
+            step0=points[:]
+            step1=[[0,0]]*3*len(step0)
+            prev=[step0[0][0],0]
+            for x in range(len(step0)):
+                step1[3*x]=[step0[x][0],prev[1]]
+                step1[3*x+1]=[step0[x][0],0.0]
+                step1[3*x+2]=step0[x]
+                prev=step0[x]
+            deltax=step0[1][0]-step0[0][0]
+            step1.append([prev[0]+deltax,prev[1]])
+            step1.append([prev[0]+deltax,0])
+            #treat monitor case
+            try: #if it is like a Monitor, take xlab,ylab from it
+                ylab=points.ylab
+                xlab=points.tlab
+                if not title: title=points.name 
+            except:
+                pass
+            #draw the line
+            smooth=False
+            return self.plotLine(step1,windowsize=windowsize,title=title,width=width,
                              color=color,smooth=smooth,background=background,
                              xlab=xlab,ylab=ylab,xaxis=(step1[0][0],step1[-1][0]),
                              yaxis=yaxis)
+        else:
+            print "SimPlot.plotHistogram: dataset empty, no plot."
+            return None
     
     def plotBars(self,points,windowsize=(500,300),title='',color='black',
                  width=1,size=3,fillcolor='black',fillstyle='',
                  outline='black',background='white',xlab='x',ylab='y',
                  xaxis='automatic',yaxis='automatic',anchor=0.0):
-
-        root=Toplevel()
-        f=Frame(root)
-        try: #if it is like a Monitor, take xlab,ylab from it
-            ylab=points.ylab
-            xlab=points.tlab
-            if not title: title=points.name 
-        except:
-            pass
-        bars=self.makeBars(points, width=width,size=size,color=color,
+        """Generates a bar chart, with menu to save as Postscript file.
+        'points' can be a Monitor instance.
+        """
+        if points!=[]:
+            root=Toplevel()
+            f=Frame(root)
+            try: #if it is like a Monitor, take xlab,ylab from it
+                ylab=points.ylab
+                xlab=points.tlab
+                if not title: title=points.name 
+            except:
+                pass
+            bars=self.makeBars(points, width=width,size=size,color=color,
                            fillcolor=fillcolor,fillstyle=fillstyle,
                            outline=outline,anchor=anchor)
-        gr=self.makeGraphObjects([bars])
-        graph = self.makeGraphBase(f, windowsize[0],windowsize[1],
+            gr=self.makeGraphObjects([bars])
+            graph = self.makeGraphBase(f, windowsize[0],windowsize[1],
                                    title=title,xtitle=xlab,
                                    ytitle=ylab,background=background)
-        graph.pack(side=LEFT, fill=BOTH, expand=YES)
-        graph.draw(gr,xaxis=xaxis, yaxis=yaxis)
-        #File menu
-        self.graphMenu(root,graph)
-        f.pack()
-        return graph
+            graph.pack(side=LEFT, fill=BOTH, expand=YES)
+            graph.draw(gr,xaxis=xaxis, yaxis=yaxis)
+            #File menu
+            self.graphMenu(root,graph)
+            f.pack()
+            return graph
+        else:
+            print "SimPlot.plotBars dataset empty, no plot."
+            return None
 
     def plotScatter(self,points,windowsize=(500,300),title='',width=1,color='black',
                     fillcolor='black',size=2,fillstyle='',
                     outline='black',marker='circle',
                     background='white',xlab='x',ylab='y',
                     xaxis='automatic',yaxis='automatic'):
-        root=Toplevel()
-        f=Frame(root)
-        try: #if it is like a Monitor, take xlab,ylab from it
-            ylab=points.ylab
-            xlab=points.tlab
-            if not title: title=points.name 
-        except:
-            pass
-        scat=self.makeSymbols(points, width=width,color=color,size=size,
+        if points!=[]:
+            root=Toplevel()
+            f=Frame(root)
+            try: #if it is like a Monitor, take xlab,ylab from it
+                ylab=points.ylab
+                xlab=points.tlab
+                if not title: title=points.name 
+            except:
+                pass
+            scat=self.makeSymbols(points, width=width,color=color,size=size,
                               marker=marker,fillcolor=fillcolor,
                               fillstyle=fillstyle,outline=outline)
-        gr=self.makeGraphObjects([scat])
-        graph = self.makeGraphBase(f, windowsize[0],windowsize[1],
+            gr=self.makeGraphObjects([scat])
+            graph = self.makeGraphBase(f, windowsize[0],windowsize[1],
                                    title=title,xtitle=xlab,
                                    ytitle=ylab,background=background)
-        graph.pack(side=LEFT, fill=BOTH, expand=YES)
-        graph.draw(gr,xaxis=xaxis, yaxis=yaxis)
-        #File menu
-        self.graphMenu(root,graph)
-        f.pack()
-        return graph
+            graph.pack(side=LEFT, fill=BOTH, expand=YES)
+            graph.draw(gr,xaxis=xaxis, yaxis=yaxis)
+            #File menu
+            self.graphMenu(root,graph)
+            f.pack()
+            return graph
+        else:
+            print "SimPlot.plotScatter: dataset empty, no plot."
+            return None
 
     def mainloop(self):
         self.root.mainloop()
