@@ -1,7 +1,15 @@
 #!/usr/bin/env python
-# testSimPyStep.py test routines
-#        $Author: kgmuller $              $Revision: 1.1.1.1.4.1.4.4 $
-#        $Date: 2005/06/06 04:32:21 $
+from __future__ import generators
+from SimPy.MonitorTest import *
+from SimPy.SimulationStep  import *
+import unittest
+from random import random
+# $Revision: 1.1.1.12 $ $Date: 2006/06/14 12:28:24 $
+"""testSimPyStep.py 
+SimPy version 1.7.1 
+Unit tests for SimulationStep.
+
+**Change history:**
 # 2002 11 15 Added tests for priority queues and preemption
 # 2002 11 22 testing problem in accum
 # 2003 03 30 added tests for SEP001v17 interrupts
@@ -15,18 +23,18 @@
 #            tests compatibility with Simulation.py
 # 2004 09 17 added tests for waitevent, queueevent, waituntil (new in 1.5)
 # 2005 05 19 added tests for compound yield statements (reneging)
-# '$Revision: 1.1.1.1.4.1.4.4 $ $Date: 2005/06/06 04:32:21 $ kgm'
-
-"""Testing SimulationStep compatibility with Simulation
+# 2006 01 15 added tests for Store and Level and the get/put yield statements
+# 2006 02 02 removed histogram plotting suite
+# 2006 05 10 changed test testStatic for Level to test that float type 
+             supported for initialBuffered
+# 2006 05 16 added tests for Store and Level to test basic Producer/Consumer 
+             principles
 """
-__version__ = '1.6 June 2005'
-Revision = '$Revision: 1.1.1.1.4.1.4.4 $'
+__version__ = '1.7.1 Jun 2006'
+Revision = '$Revision: 1.1.1.12 $'
+
 
-from __future__ import generators
-from SimPy.MonitorTest import *
-from SimPy.SimulationStep  import *
 
-import unittest
 
 ## -------------------------------------------------------------
 ##                    TEST SIMULATION
@@ -151,7 +159,7 @@ class makeResourcetestcase(unittest.TestCase):
         activate(J2,J2.execute(), at=0.0) # requests a unit of R2
         ## when simulation starts
         simulate(until = 10.0)
-        assert R2.n == 0 , "Should be 0, it is "+str(R1.n)
+        assert R2.n == 0 , "Should be 0, it is "+str(R2.n)
         lenW = len(R2.waitQ)
         lenA = len(R2.activeQ)
         assert lenW==0,"lenW Should be 0, it is "+str(lenW)
@@ -170,7 +178,7 @@ class makeResourcetestcase(unittest.TestCase):
         activate(J4,J4.execute(), at=0.0) # requests a unit of R3
         ## when simulation starts
         simulate(until = 10.0)
-        assert R3.n == 0 , "Should be 0, it is "+str(R13.n)
+        assert R3.n == 0 , "Should be 0, it is "+str(R3.n)
         lenW = len(R3.waitQ)
         lenA = len(R3.activeQ)
         assert lenW==2,"lenW Should be 2, it is "+str(lenW)
@@ -191,7 +199,7 @@ class makeResourcetestcase(unittest.TestCase):
         activate(J4,J4.execute(), at=0.0) # requests a unit of R3
         ## when simulation starts
         simulate(until = 10.0)
-        assert R3.n == 0 , "Should be 0, it is "+str(R13.n)
+        assert R3.n == 0 , "Should be 0, it is "+str(R3.n)
         lenW = len(R3.waitQ)
         lenA = len(R3.activeQ)
         assert lenW==1,"lenW Should be 1, it is "+str(lenW)
@@ -225,7 +233,7 @@ class makeResourcetestcase(unittest.TestCase):
         assert Rp.waitQ == [J5,J7,J6],"WaitQ wrong"+str([(x.name,x.priority[Rp]) for x in Rp.waitQ])
 
         """Test PriorityQ mechanism"""
-        from random import random
+
         def sorted(q):
            if not q or len(q) == 1:
               sortok=1
@@ -715,7 +723,7 @@ def makePSuite():
    testState3=makePStatetestcase("testState3")
    testState4=makePStatetestcase("testState4")
    suite.addTests([testState1,testState2,testState3,testState4])
-   return suite 
+   return suite
 
 ## -------------------------------------------------------------
 ##                    TEST Events/Signals
@@ -758,7 +766,7 @@ class WaitProcessOR(Process):
       yield waitevent,self,evset
       for e in evset:
          assert e.waits==[],"process not out of waiting list for all events in OR"
-      
+         
 class WaitProcessOR1(Process):
     def signalandwait(self):
       e1=SimEvent()
@@ -805,7 +813,7 @@ class makeEtestcase(unittest.TestCase):
       assert e.occurred,"signal does not set 'occurred' to True"
       assert e.signalparam=="param","signal parameter wrong"
       e.signal()
-      assert e.signalparam==None,"signal with no parameter did not overwrite signalparam"
+      assert e.signalparam is None,"signal with no parameter did not overwrite signalparam"
       e.signal()
       assert e.occurred,"multiple calls to signal do not set 'occurred'"
       
@@ -840,7 +848,7 @@ class makeEtestcase(unittest.TestCase):
       for i in range(5):
          q=QueueProcessOR()
          activate(q,q.queueForSig([e2,e3]))
-      simulate(until=10)    
+      simulate(until=10) 
 
    def testSimEvents4(self):
       """Tests that eventsFired reports all events which fired
@@ -1118,6 +1126,8 @@ def makeTOSuite():
 ##                   == event renege
 ##             for both unmonitored and monitored resources
 ## ------------------------------------------------------------------
+
+
 class JobEvt(Process):
    """ Job class for testing event reneging
    """
@@ -1313,13 +1323,448 @@ def makeEvtRenegeSuite():
     suite.addTests([testNoEvent,testNoEventM,testWaitEvent1,testWaitEvent1M,
                     testWaitEvent2,testWaitEvent2M]) 
     return suite
+    
+#---Buffer tests (post 1.6.1)-------------------------------------
+## ------------------------------------------------------------------
+##             TEST "yield get,self,level,whatToGet" and 
+##                  "yield put,self,level,whatToPut,priority"
+##             for Level instances
+## ------------------------------------------------------------------
+class Producer(Process):
+    produced=0
+    def produce(self,buffer):
+        for i in range(4):
+            Producer.produced+=1
+            yield put,self,buffer
+            yield hold,self,1
+    def producePriority(self,buffer,priority):
+        """PriorityQ for Producers"""
+        Producer.produced+=4
+        yield put,self,buffer,4,priority
+        yield hold,self,1
+        self.done=now()
+        doneList.append(self.name)
+    def produce1(self,buffer):
+        for i in range(4):
+            yield put,self,buffer,4
+            yield hold,self,1
+class Consumer(Process):
+    consumed=0
+    def consume(self,buffer):
+        """FIFO"""
+        yield get,self,buffer
+        Consumer.consumed+=1
+        assert self.got==1,"wrong self.got: %s"%self.got 
+        yield get,self,buffer,3
+        Consumer.consumed+=3
+        assert self.got==3,"wrong self.got: %s"%self.got
+        
+    def consume1(self,buffer):
+        """producer PriorityQ, consumer FIFO"""
+        while True:
+            yield get,self,buffer,2
+            yield hold,self,1
+    def consumePriority(self,buffer,priority):
+        """PriorityQ for Consumers"""
+        yield get,self,buffer,4,priority
+        doneList.append(self.name)
+        
+### Begin classes for testConPrinciple (Level) ###
+class  ProducerPrincL(Process):
+    def produce(self,buffer,productionTime):     
+        while True:
+            assert not(buffer.amount>0 and len(buffer.getQ)>0),\
+                "Consumer(s) waiting while buffer not empty"
+            yield hold,self,productionTime
+            yield put,self,buffer,1
+
+class ConsumerPrincL(Process):
+    def consume(self,buffer,consumptionTime):
+        while True:
+            assert not(buffer.amount==0 and len(buffer.putQ)>0),\
+                "Producer(s) waiting while buffer empty"
+            yield get,self,buffer,1
+            yield hold,self,consumptionTime        
+                
+### End classes for testConPrinciple (Level) ###
+
+class makeLevelTestcase(unittest.TestCase):
+    def testStatic(self):
+        """Tests initialization of Level instances
+        """
+        a=Level()
+        assert a.capacity==sys.maxint,"wrong capacity:%s"%a
+        assert a.amount==0,"wrong buffer content: %s"%a
+        assert a.name=="a_level","wrong name: %s"%a
+        assert not a.monitored,"should not be monitored: %s"%a
+        assert a.putQMon is None,"should not have putQMon: %s"%a
+        assert a.getQMon is None,"should not have getQMon: %s"%a
+        assert a.bufferMon is None,"should not have bufferMon: %s"%a
+        assert a.putQType.__name__=="FIFO" and a.getQType.__name__=="FIFO",\
+               "putQType and getQType should be FIFO: %s"%a
+        
+        b=Level(name="b",initialBuffered=10.0,monitored=True,capacity=12,
+                    putQType=PriorityQ)
+        a=Level()
+        assert b.capacity==12,"wrong capacity:%s"%b
+        assert b.amount==10,"wrong buffer content: %s"%b
+        assert b.name=="b","wrong name: %s"%b
+        assert b.monitored,"should be monitored: %s"%b
+        assert not (b.putQMon is None),"should have putQMon: %s"%b
+        assert not (b.getQMon is None),"should have getQMon: %s"%b
+        assert not (b.bufferMon is None),"should have bufferMon: %s"%b
+        assert b.putQType.__name__=="PriorityQ",\
+               "putQType should be PriorityQ: %s"%b
+        assert b.getQType.__name__=="FIFO",\
+                "getQType should be PriorityQ: %s"%b
+                
+    def testConProdPrinciple(self):
+        """Level: tests basic Producer/Consumer principles:
+        -   Consumers must not be waiting while Level buffer value > 0,
+        -   Producers must not be waiting while Level buffer value == 0
+        """
+        bufferSize=1
+        productionTime=1
+        consumptionTime=5
+        endtime=50
+    
+        initialize()
+        buffer=Level(capacity=bufferSize)
+        consumer=ConsumerPrincL()
+        activate(consumer,consumer.consume(buffer,consumptionTime))
+        producer=ProducerPrincL()
+        activate(producer,producer.produce(buffer,productionTime))
+        simulate(until=endtime)
+
+    def testConProd1(self):
+        """Level: tests put/get in 1 Producer/ 1 Consumer scenario"""
+        initialize()
+        buffer=Level(initialBuffered=0)
+        p=Producer()
+        activate(p,p.produce(buffer))
+        c=Consumer()
+        activate(c,c.consume(buffer))
+        simulate(until=100)
+        assert Producer.produced-Consumer.consumed==buffer.amount,\
+            "items produced/consumed/buffered do not tally: %s %s %s"\
+                %(Producer.produced,Consumer.consumed,buffer.amount)
+        
+    def testConProdM(self):
+        """Level: tests put/get in multiple Producer/Consumer scenario"""
+        initialize()
+        buffer=Level(initialBuffered=0)
+        Producer.produced=0
+        Consumer.consumed=0
+        for i in range(2):
+            c=Consumer()
+            activate(c,c.consume(buffer))
+        for i in range(3):
+            p=Producer()
+            activate(p,p.produce(buffer))
+        simulate(until=10)
+        assert Producer.produced-Consumer.consumed==buffer.amount,\
+            "items produced/consumed/buffered do not tally: %s %s %s"\
+                %(Producer.produced,Consumer.consumed,buffer.amount)
+
+    def testConProdPriorM(self):
+        """Level: tests put/get in multiple Producer/Consumer scenario, 
+        with Producers having different priorities.
+        How: Producers forced to queue; all after first should be done in
+        priority order
+        """
+        global doneList
+        doneList=[]
+        initialize()
+        buffer=Level(capacity=7,putQType=PriorityQ,monitored=True)
+        for i in range(4):
+            p=Producer(i)
+            pPriority=i
+            activate(p,p.producePriority(buffer=buffer,priority=pPriority))
+        c=Consumer()
+        activate(c,c.consume1(buffer=buffer))
+        simulate(until=100)
+        assert doneList==[0,3,2,1],"puts were not done in priority order: %s"\
+                                    %doneList
+                                    
+    def testConPriorProdM(self):
+        """Level: ests put/get in multiple Producer/Consumer scenario, with
+        Consumers having different priorities.
+        How: Consumers forced to queue; all after first should be done in
+        priority order
+        """
+        global doneList
+        doneList=[]
+        initialize()
+        buffer=Level(capacity=7,getQType=PriorityQ,monitored=True)
+        for i in range(4):
+            c=Consumer(i)
+            cPriority=i
+            activate(c,c.consumePriority(buffer=buffer,priority=cPriority))
+        p=Producer()
+        activate(p,p.produce1(buffer=buffer))
+        simulate(until=100)
+        assert doneList==[3,2,1,0],"gets were not done in priority order: %s"\
+                                    %doneList
+
+def makeLevelSuite():
+    suite = unittest.TestSuite()
+    testStatic = makeLevelTestcase("testStatic")
+    testConProdPrinciple=makeLevelTestcase("testConProdPrinciple")
+    testConProd1=makeLevelTestcase("testConProd1")
+    testConProdM=makeLevelTestcase("testConProdM")
+    testConProdPriorM=makeLevelTestcase("testConProdPriorM")
+    testConPriorProdM=makeLevelTestcase("testConPriorProdM")
+    suite.addTests([testStatic,testConProdPrinciple,testConProd1,
+                    testConProdM,testConProdPriorM,
+                    testConPriorProdM]) 
+    return suite
+
+
+## ------------------------------------------------------------------
+##             TEST "yield get,self,store,whatToGet" and 
+##                  "yield put,self,store,whatToPut"
+##             for Store instances
+## ------------------------------------------------------------------
+
+class ProducerWidget(Process):
+    produced=0
+    def produce(self,buffer):
+        for i in range(4):
+            ProducerWidget.produced+=1
+            yield put,self,buffer,[Widget(weight=5)]
+            yield hold,self,1
+    def producePriority(self,buffer,priority):
+        """PriorityQ for Producers"""
+        ProducerWidget.produced+=4
+        toStore=[Widget(weight=5)]*4
+        yield put,self,buffer,toStore,priority
+        yield hold,self,1
+        self.done=now()
+        doneList.append(self.name)
+    def produce1(self,buffer):
+        for i in range(4):
+            yield put,self,buffer,[Widget(weight=5)]*4
+            yield hold,self,1
+    def produceUnordered(self,buffer):
+        produced=[Widget(weight=i) for i in [9,1,8,2,7,3,6,4,5]]
+        yield put,self,buffer,produced
+
+class ConsumerWidget(Process):
+    consumed=0
+    def consume(self,buffer):
+        """FIFO"""
+        yield get,self,buffer
+        ConsumerWidget.consumed+=1
+        assert len(self.got)==1,"wrong self.got: %s"%self.got 
+        yield get,self,buffer,3
+        ConsumerWidget.consumed+=3
+        assert len(self.got)==3,"wrong self.got: %s"%self.got
+        
+    def consume1(self,buffer):
+        """producer PriorityQ, consumer FIFO"""
+        while True:
+            yield get,self,buffer,2
+            yield hold,self,1
+        
+    def consumePriority(self,buffer,priority):
+        """PriorityQ for Consumers"""
+        yield get,self,buffer,4,priority
+        doneList.append(self.name)
+        
+    def consumeSorted(self,buffer,gotten):
+        yield get,self,buffer
+        gotten.append(self.got[0].weight)
+
+class Widget:
+    def __init__(self,weight):
+        self.weight=weight
+        
+def mySortFunc(self,par):
+    """Sorts Widget instances by weight attribute."""
+    tmplist=[(x.weight,x) for x in par]
+    tmplist.sort()
+    return [x for (key,x) in tmplist]
+    
+### Begin classes for testConPrinciple (Store) ###
+class  ProducerPrincS(Process):
+    def produce(self,buffer,productionTime):     
+        while True:
+            assert not(buffer.nrBuffered>0 and len(buffer.getQ)>0),\
+                "Consumer(s) waiting while buffer not empty"
+            yield hold,self,productionTime
+            product=WidgetPrinc()
+            yield put,self,buffer,[product]
+
+class ConsumerPrincS(Process):
+    def consume(self,buffer,consumptionTime):
+        while True:
+            assert not(buffer.nrBuffered==0 and buffer.putQ),\
+                "Producer(s) waiting while buffer empty"
+            yield get,self,buffer,1
+            yield hold,self,consumptionTime        
+                
+class WidgetPrinc:
+    pass
+
+### End classes for testConPrinciple (Store) ###
+
+class makeStoreTestcase(unittest.TestCase):
+    def testStatic(self):
+        """Store: tests initialization of Store instances
+        """
+        a=Store()
+        assert a.capacity==sys.maxint,"wrong capacity:%s"%a
+        assert a.nrBuffered==0,"wrong buffer content: %s"%a
+        assert a.name=="a_store","wrong name: %s"%a
+        assert not a.monitored,"should not be monitored: %s"%a
+        assert a.putQMon is None,"should not have putQMon: %s"%a
+        assert a.getQMon is None,"should not have getQMon: %s"%a
+        assert a.bufferMon is None,"should not have bufferMon: %s"%a
+        assert a.putQType.__name__=="FIFO" and a.getQType.__name__=="FIFO",\
+               "putQType and getQType should be FIFO: %s"%a
+            
+        stored=[Widget(weight=5)]*10       
+        b=Store(name="b",initialBuffered=stored,monitored=True,capacity=12,
+                    putQType=PriorityQ)
+        assert b.capacity==12,"wrong capacity:%s"%b
+        assert b.nrBuffered==10,"wrong buffer content: %s"%b
+        assert b.name=="b","wrong name: %s"%b
+        assert b.monitored,"should be monitored: %s"%b
+        assert not (b.putQMon is None),"should have putQMon: %s"%b
+        assert not (b.getQMon is None),"should have getQMon: %s"%b
+        assert not (b.bufferMon is None),"should have bufferMon: %s"%b
+        assert b.putQType.__name__=="PriorityQ",\
+               "putQType should be PriorityQ: %s"%b
+        assert b.getQType.__name__=="FIFO",\
+                "getQType should be PriorityQ: %s"%b
+
+    def testConProdPrinciple(self):
+        """Store: tests basic Producer/Consumer principles:
+        -   Consumers must not be waiting while items in Store buffer,
+        -   Producers must not be waiting while space available in Store buffer
+        """
+        bufferSize=1
+        productionTime=1
+        consumptionTime=5
+        endtime=50
+
+        initialize()
+        buffer=Store(capacity=bufferSize)
+        consumer=ConsumerPrincS()
+        activate(consumer,consumer.consume(buffer,consumptionTime))
+        producer=ProducerPrincS()
+        activate(producer,producer.produce(buffer,productionTime))
+        simulate(until=endtime)
+
+    def testConProd1(self):
+        """Store: tests put/get in 1 Producer/ 1 Consumer scenario"""
+        initialize()
+        buffer=Store(initialBuffered=[])
+        p=ProducerWidget()
+        activate(p,p.produce(buffer))
+        c=ConsumerWidget()
+        activate(c,c.consume(buffer))
+        simulate(until=100)
+        assert ProducerWidget.produced-ConsumerWidget.consumed==buffer.nrBuffered,\
+            "items produced/consumed/buffered do not tally: %s %s %s"\
+            %(ProducerWidget.produced,ConsumerWidget.consumed,buffer.nrBuffered)
+        
+    def testConProdM(self):
+        """Store: tests put/get in multiple Producer/Consumer scenario"""
+        initialize()
+        buffer=Store(initialBuffered=[])
+        ProducerWidget.produced=0
+        ConsumerWidget.consumed=0
+        for i in range(2):
+            c=ConsumerWidget()
+            activate(c,c.consume(buffer))
+        for i in range(3):
+            p=ProducerWidget()
+            activate(p,p.produce(buffer))
+        simulate(until=10)
+        assert ProducerWidget.produced-ConsumerWidget.consumed==buffer.nrBuffered,\
+            "items produced/consumed/buffered do not tally: %s %s %s"\
+            %(ProducerWidget.produced,ConsumerWidget.consumed,buffer.nrBuffered)
+
+    def testConProdPriorM(self):
+        """Store: Tests put/get in multiple Producer/Consumer scenario, 
+        with Producers having different priorities.
+        How; Producers forced to queue; all after first should be done in
+        priority order
+        """
+        global doneList
+        doneList=[]
+        initialize()
+        buffer=Store(capacity=7,putQType=PriorityQ,monitored=True)
+        for i in range(4):
+            p=ProducerWidget(i)
+            pPriority=i
+            activate(p,p.producePriority(buffer=buffer,priority=pPriority))
+        c=ConsumerWidget()
+        activate(c,c.consume1(buffer=buffer))
+        simulate(until=100)
+        assert doneList==[0,3,2,1],"puts were not done in priority order: %s"\
+                                    %doneList
+                                    
+    def testConPriorProdM(self):
+        """Tests put/get in multiple Producer/Consumer scenario, with
+        Consumers having different priorities.
+        How; Consumers forced to queue; all after first should be done in
+        priority order
+        """
+        global doneList
+        doneList=[]
+        initialize()
+        buffer=Store(capacity=7,getQType=PriorityQ,monitored=True)
+        for i in range(4):
+            c=ConsumerWidget(str(i))
+            cPriority=i
+            activate(c,c.consumePriority(buffer=buffer,priority=cPriority))
+        p=ProducerWidget()
+        activate(p,p.produce1(buffer=buffer))
+        simulate(until=100)
+        assert doneList==["3","2","1","0"],"gets were not done in priority order: %s"\
+                                    %doneList
+                                    
+    def testBufferSort(self):
+        """Tests the optional sorting of theBuffer by applying a user-defined
+        sort function."""
+        initialize()
+        gotten=[]
+        sortedStore=Store()
+        sortedStore.addSort(mySortFunc)
+        p=ProducerWidget()
+        activate(p,p.produceUnordered(sortedStore))
+        for i in range(9):
+            c=ConsumerWidget()
+            activate(c,c.consumeSorted(buffer=sortedStore,gotten=gotten),at=1)
+        simulate(until=10)
+        assert gotten==[1,2,3,4,5,6,7,8,9],"sort wrong: %s"%gotten
+        
+def makeStoreSuite():
+    suite = unittest.TestSuite()
+    testStatic = makeStoreTestcase("testStatic")
+    testConProdPrinciple=makeStoreTestcase("testConProdPrinciple")
+    testConProd1=makeStoreTestcase("testConProd1")
+    testConProdM=makeStoreTestcase("testConProdM")
+    testConProdPriorM=makeStoreTestcase("testConProdPriorM")
+    testConPriorProdM=makeStoreTestcase("testConPriorProdM")
+    testBufferSort=makeStoreTestcase("testBufferSort")
+    suite.addTests([testStatic,testConProdPrinciple,testConProd1,
+                    testConProdM,testConProdPriorM,
+                    testConPriorProdM,testBufferSort]) 
+    return suite
+
 
 if __name__ == '__main__':
     alltests = unittest.TestSuite((makeSSuite(),makeRSuite(),
-                                   makeMSuite(),makeHSuite(),
+                                   makeMSuite(),#makeHSuite(),
                                    makeISuite(),makePSuite(),
                                    makeESuite(),makeWSuite(),
-                                   makeTOSuite(),makeEvtRenegeSuite()))
+                                   makeTOSuite(),makeEvtRenegeSuite(),
+                                   makeLevelSuite(),
+                                   makeStoreSuite()
+                                ))
     runner = unittest.TextTestRunner()
     runner.run(alltests)
 
